// $Id: reloadeveryOverlay.js,v 1.5 2006/02/20 21:44:08 jaap Exp $

/* ***** BEGIN LICENSE BLOCK ***** 
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Initial Developer of the Original Code is Jaap Haitsma.
 * Portions created by the Initial Developer are Copyright (C) 2004
 * by the Initial Developer. All Rights Reserved.
 *
 * Contributor(s): Jaap Haitsma <jaap@haitsma.org>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

// Uncomment DEBUG part and //log statements to have logging to the console
// Furthermore you need to add the following two lines to prefs.js to see
// the logging in the shell where you started the browser
// user_pref("browser.dom.window.dump.enabled", true);
// user_pref("javascript.options.showInConsole", true);

////////////////////////////////////////////////////////////////////////////
// Debug part
/////////////////////////////////////////////////////////////////////////////
/*
function //log(str){
  dump(str + "\n");
}

function logObject(obj){
  dump("\nlogging object contents\n");
  for(i in obj){
    dump(i + " = " + obj[i] + "\n");
  }
  dump("\n");
}
*/
////////////////////////////////////////////////////////////////////////////


const DEFAULT_RELOAD_TIME = 10;
const DEFAULT_CUSTOM_RELOAD_TIME = 90;

// Every time a new browser window is made reloadEveryInit will be called
window.addEventListener("load",reloadEveryInit,false);


// This holds the object for accessing the preferences stored in prefs.js
var reloadEveryPrefs = Components.classes["@mozilla.org/preferences-service;1"].
                       getService(Components.interfaces.nsIPrefService).getBranch("reloadevery.");


function getCurTab(){
    return getBrowser().mCurrentBrowser;
}


var gReloadEveryDialogAccepted;

function reloadEveryDialogResult(accepted){
    //log("reloadEveryDialogResult(): accepted = " + accepted );

    gReloadEveryDialogAccepted = accepted;
}


function reloadEveryInit() {
    //log("reloadEveryInit()"); 

    if (!reloadEveryPrefs.prefHasUserValue("reload_time")){
        //log("reloadEveryInit(): no preferences found in prefs.js taking default for reload_time");
        // Assign default value
        reloadEveryPrefs.setIntPref("reload_time", DEFAULT_RELOAD_TIME);
    }

    if (!reloadEveryPrefs.prefHasUserValue("custom_reload_time")){
        //log("reloadEveryInit(): no preferences found in prefs.js taking default for custom_reload_time");
        // Assign default value
        reloadEveryPrefs.setIntPref("custom_reload_time", DEFAULT_CUSTOM_RELOAD_TIME);
    }

    try{
	document.getElementById("contentAreaContextMenu").addEventListener("popupshowing",onReloadEveryPopup,false);
        gURLBar.addEventListener("keypress", onReloadEveryKeyPressInURLBar, false);
    }
    catch(e){
        // Do nothing. reloadEveryInit() is also called when the preferences dialog is called and there 
        // gURLBar.addEventListener("keypress", onReloadEveryKeyPressInURLBar, false);  
        // fails
    }
}


var reloadEveryTabID=0;

function setupReloadEvery(tab){
    //log("setupReloadEvery(tab)");

    // Add member attributes
    tab.reloadEveryEnabled = false;
    tab.reloadEveryReloadTime = reloadEveryPrefs.getIntPref("reload_time");
    //log("reload time: " + tab.reloadEveryReloadTime);
    tab.reloadEveryTimerID = null;
    tab.postDataAcceptedByUser = false;
    tab.id = "ActiveReloadTab" + reloadEveryTabID;     
    reloadEveryTabID++;    

    tab.reloadEveryProgressListener = reloadEveryProgressListener(tab);
}


// hide the Reload Every item when apropriate (use same logic as for Back, Stop etc.)
function onReloadEveryPopup() {

    //log("onReloadEveryPopup()");
    var cm = gContextMenu;
    document.getElementById("reloadevery_menu").hidden = ( cm.isTextSelected || cm.onLink || cm.onImage || cm.onTextInput );

    // Check if this a new window/tab   
    if (getCurTab().reloadEveryEnabled == null){
        //log("onReloadEveryPopup() new window"); 
        setupReloadEvery(getCurTab());    
    }

    if (getCurTab().reloadEveryEnabled){
        document.getElementById("reloadevery_enable").setAttribute("checked", "true");
    }
    else{
        document.getElementById("reloadevery_enable").setAttribute("checked", "false");
    }
 
    // First uncheck all radio menuitems
    document.getElementById("reloadevery_5s").setAttribute("checked", "false");
    document.getElementById("reloadevery_10s").setAttribute("checked", "false");
    document.getElementById("reloadevery_30s").setAttribute("checked", "false");
    document.getElementById("reloadevery_1m").setAttribute("checked", "false");
    document.getElementById("reloadevery_5m").setAttribute("checked", "false");
    document.getElementById("reloadevery_15m").setAttribute("checked", "false");
    document.getElementById("reloadevery_custom").setAttribute("checked", "false");

    // Now select the appropriate one
 
    if (getCurTab().reloadEveryReloadTime == 5){
        document.getElementById("reloadevery_5s").setAttribute("checked", "true");
    }
    else if (getCurTab().reloadEveryReloadTime == 10){
        document.getElementById("reloadevery_10s").setAttribute("checked", "true");
    }
    else if (getCurTab().reloadEveryReloadTime == 30){
        document.getElementById("reloadevery_30s").setAttribute("checked", "true");
    }
    else if (getCurTab().reloadEveryReloadTime == 60){
        document.getElementById("reloadevery_1m").setAttribute("checked", "true");  
    }
    else if (getCurTab().reloadEveryReloadTime == 5*60){
        document.getElementById("reloadevery_5m").setAttribute("checked", "true");
    }
    else if (getCurTab().reloadEveryReloadTime == 15*60){
        document.getElementById("reloadevery_15m").setAttribute("checked", "true");
    }
    else if (getCurTab().reloadEveryReloadTime >= 0){
        document.getElementById("reloadevery_custom").setAttribute("checked", "true"); 
    }
    else{
        alert ("Invalid reload time:" + getCurTab().reloadEveryReloadTime + "s");
    }
}


// hide the Reload Every item when apropriate (use same logic as for Back, Stop etc.)
function onButReloadEveryPopup() {

    //log("onButReloadEveryPopup()");

    // Check if this a new window/tab   
    if (getCurTab().reloadEveryEnabled == null){
        //log("onReloadEveryPopup() new window"); 
        setupReloadEvery(getCurTab());    
    }

    if (getCurTab().reloadEveryEnabled){
        document.getElementById("but_reloadevery_enable").setAttribute("checked", "true");
    }
    else{
        document.getElementById("but_reloadevery_enable").setAttribute("checked", "false");
    }
 
    // First uncheck all radio menuitems
    document.getElementById("but_reloadevery_5s").setAttribute("checked", "false");
    document.getElementById("but_reloadevery_10s").setAttribute("checked", "false");
    document.getElementById("but_reloadevery_30s").setAttribute("checked", "false");
    document.getElementById("but_reloadevery_1m").setAttribute("checked", "false");
    document.getElementById("but_reloadevery_5m").setAttribute("checked", "false");
    document.getElementById("but_reloadevery_15m").setAttribute("checked", "false");
    document.getElementById("but_reloadevery_custom").setAttribute("checked", "false");

    // Now select the appropriate one
 
    if (getCurTab().reloadEveryReloadTime == 5){
        document.getElementById("but_reloadevery_5s").setAttribute("checked", "true");
    }
    else if (getCurTab().reloadEveryReloadTime == 10){
        document.getElementById("but_reloadevery_10s").setAttribute("checked", "true");
    }
    else if (getCurTab().reloadEveryReloadTime == 30){
        document.getElementById("but_reloadevery_30s").setAttribute("checked", "true");
    }
    else if (getCurTab().reloadEveryReloadTime == 60){
        document.getElementById("but_reloadevery_1m").setAttribute("checked", "true");  
    }
    else if (getCurTab().reloadEveryReloadTime == 5*60){
        document.getElementById("but_reloadevery_5m").setAttribute("checked", "true");
    }
    else if (getCurTab().reloadEveryReloadTime == 15*60){
        document.getElementById("but_reloadevery_15m").setAttribute("checked", "true");
    }
    else if (getCurTab().reloadEveryReloadTime >= 0){
        document.getElementById("but_reloadevery_custom").setAttribute("checked", "true"); 
    }
    else{
        alert ("Invalid reload time:" + getCurTab().reloadEveryReloadTime + "s");
    }
}



// Reloads the page of the tab with the specified reloadEveryTabID 
function reloadPage(reloadEveryTabID){
    //log("reloadPage(...) : " + reloadEveryTabID);
    tab = document.getElementById(reloadEveryTabID); 

    if (tab == null){
        //log("reloadPage(...) : ReloadEvery disabled");
        return;
    }

    if (tab.reloadEveryEnabled == false){
        tab.postDataAcceptedByUser = false;
        //log("reloadPage(...) : ReloadEvery disabled");
        return;
    }

    //log("reloadPage(...) : " + reloadEveryTabID + "reloading url :" + tab.webNavigation.currentURI.spec);
    var loadFlags = nsIWebNavigation.LOAD_FLAGS_BYPASS_HISTORY | nsIWebNavigation.LOAD_FLAGS_BYPASS_PROXY | nsIWebNavigation.LOAD_FLAGS_BYPASS_CACHE;
    var entry=tab.webNavigation.sessionHistory.getEntryAtIndex(tab.webNavigation.sessionHistory.index, false);
    var postData = entry.QueryInterface(Components.interfaces.nsISHEntry).postData;
    var referrer = entry.QueryInterface(Components.interfaces.nsISHEntry).referrerURI;
	
    if ((postData!=null) && (tab.postDataAcceptedByUser==false)){
        window.openDialog("chrome://reloadevery/content/warnPostData.xul", "",
                          "chrome,centerscreen,modal", reloadEveryDialogResult);
        if (gReloadEveryDialogAccepted){
            //log("reloadPage(...) : POSTDATA accepted");
            tab.postDataAcceptedByUser = true;
        }
        else{
            //log("reloadPage(...) : POSTDATA not accepted");
            tab.reloadEveryEnabled=false;
            return;           
        }        
    }                      
    tab.curScrollX = tab.contentWindow.scrollX;
    tab.curScrollY = tab.contentWindow.scrollY;
    //log("Current scroll position " + tab.curScrollX + ", "+ tab.curScrollY);

    var notifyFlags = Components.interfaces.nsIWebProgress.NOTIFY_ALL;
    tab.webProgress.addProgressListener(tab.reloadEveryProgressListener, notifyFlags);         
    tab.webNavigation.loadURI(tab.webNavigation.currentURI.spec, loadFlags, referrer, entry.postData, null);                       
}

function enableReloadEvery(tab){
    //log("enableReloadEvery(tab)");
    tab.reloadEveryEnabled = true;
    clearInterval(tab.reloadEveryTimerID);
    tab.reloadEveryTimerID=setTimeout("reloadPage(\"" + tab.id + "\");", tab.reloadEveryReloadTime*1000);  
}

function disableReloadEvery(tab){
    //log("disableReloadEvery(tab)");
    tab.reloadEveryEnabled = false;
    tab.postDataAcceptedByUser = false;
    clearInterval(tab.reloadEveryTimerID);
}

///////////////////////////////////////////////////////////////////////////////

function onReloadEveryEnable(){ 
    //log("onReloadEveryEnable()");

    if (getCurTab().reloadEveryEnabled){
        disableReloadEvery(getCurTab());
    }
    else{
        enableReloadEvery(getCurTab());    
    }
}

function onSetReloadTime(aReloadTime){
    //log ("onSetReloadTime() ReloadTime " + aReloadTime);

    getCurTab().reloadEveryReloadTime=aReloadTime;
    reloadEveryPrefs.setIntPref("reload_time", getCurTab().reloadEveryReloadTime);    
    enableReloadEvery(getCurTab());
}


function onReloadEveryCustom(){
    //log("onReloadEveryCustom()");
    
    window.openDialog("chrome://reloadevery/content/reloadeveryCustomDialog.xul", "",
                      "chrome,centerscreen,modal", reloadEveryDialogResult);
    if (gReloadEveryDialogAccepted){
       //log("onReloadEveryCustom() accepted");
        getCurTab().reloadEveryReloadTime=reloadEveryPrefs.getIntPref("custom_reload_time");         
        enableReloadEvery(getCurTab());

    }
}

function reloadeveryCustomDialogLoadSettings(){
    //log("reloadeveryCustomDialogLoadSettings()");

    var customReloadTime = reloadEveryPrefs.getIntPref("custom_reload_time");

    document.getElementById("reload_every_minutes").value = Math.floor(customReloadTime / 60);
    document.getElementById("reload_every_seconds").value = customReloadTime % 60;

}

function reloadeveryCustomDialogSaveSettings(){
    //log("reloadeveryCustomDialogSaveSettings()");
 
    var minutes;
    if(document.getElementById("reload_every_minutes").value != ''){
      minutes = parseInt(document.getElementById("reload_every_minutes").value);
    }
    else{
      minutes = 0;
    }

    var seconds;
    if(document.getElementById("reload_every_seconds").value != ''){
      seconds = parseInt(document.getElementById("reload_every_seconds").value);
    }
    else{
      seconds = 0;
    }

    var customReloadTime = minutes*60 + seconds;
    reloadEveryPrefs.setIntPref("custom_reload_time", customReloadTime);
    reloadEveryPrefs.setIntPref("reload_time", customReloadTime);
    
    return true;
}


function onReloadEveryEnableAllTabs(){ 
    //log("onReloadEveryEnableAllTabs()");
    
    nTabs=getBrowser().browsers.length;

    for(var i=0; i<nTabs; i++){
       var tab = getBrowser().browsers[i];

       if (tab.reloadEveryEnabled == null){
            setupReloadEvery(tab);
       }
       
       if (tab.reloadEveryEnabled != true){
            enableReloadEvery(tab);
       }       
    }
}


function onReloadEveryDisableAllTabs(){ 
    //log("onReloadEveryDisableAllTabs()");
    
    nTabs=getBrowser().browsers.length;

    for(var i=0; i<nTabs; i++){
       var tab = getBrowser().browsers[i];
       
       if (tab.reloadEveryEnabled == true){
            disableReloadEvery(tab);
       }       
    }
}

///////////////////////////////////////////////////////////////////////////////

// If user changes URL disable reloadevery
function onReloadEveryKeyPressInURLBar(){
    if (getCurTab().reloadEveryEnabled){
        //log("onReloadEveryKeyPressInURLBar(): disabling reload every");
        getCurTab().reloadEveryEnabled = false;
        getCurTab().postDataAcceptedByUser = false;
        clearInterval(getCurTab().reloadEveryTimerID);
    }
}


function reloadEveryProgressListener(tab){
    return ({

    QueryInterface : function(aIID){        
		if (aIID.equals(Components.interfaces.nsIWebProgressListener) ||
		    aIID.equals(Components.interfaces.nsISupportsWeakReference) ||
			aIID.equals(Components.interfaces.nsISupports)){
			return this;
        }
		throw Components.results.NS_NOINTERFACE;
	},    
    onProgressChange : function (aWebProgress, aRequest, aCurSelfProgress, aMaxSelfProgress,
                                 aCurTotalProgress, aMaxTotalProgress){
    },

    onStateChange : function(aWebProgress, aRequest, aStateFlags, aStatus){

        if ((aStateFlags & Components.interfaces.nsIWebProgressListener.STATE_IS_WINDOW) && (aStateFlags &
        Components.interfaces.nsIWebProgressListener.STATE_STOP)){        
            //log("reloadEveryProgressListener(): page loaded " + tab.id);
            tab.webProgress.removeProgressListener(tab.reloadEveryProgressListener);
            //log("scroll to " + tab.curScrollX + "," + tab.curScrollY);
	    tab.contentWindow.scrollTo(tab.curScrollX, tab.curScrollY); 
            tab.reloadEveryTimerID=setTimeout("reloadPage(\"" + tab.id + "\");", tab.reloadEveryReloadTime*1000);
        }
    },

    onLocationChange : function(aWebProgress, aRequest, aLocation){
    },

    onStatusChange : function(aWebProgress, aRequest, aStatus, aMessage){
    },

    onSecurityChange : function(aWebProgress, aRequest, aState){
    }
    });		
}
